{% extends 'gfiles/simple_form.html' %}
{% load static %}
{% block sb_admin_title %}
    {% if form.instance.pk %}
        Update registered Galaxy user
    {% else %}
        Register Galaxy user
    {% endif %}
{% endblock sb_admin_title %}
{% block sb_admin_subheading %}{% endblock sb_admin_subheading %}
{% block sb_admin_breadcrumb %}
    <a href="{% url "index" %}"><i class="fa fa-fw fa-dashboard"></i> Dashboard</a>
    / <a href="{% url "galaxy_summary" %}"> Galaxy Summary</a>
    / <a href="{% url "list_galaxy_user" %}"> Galaxy Users</a>
{% endblock sb_admin_breadcrumb %}
{% block sb_admin_breadcrumb_active %}
    {% if form.instance.pk %}
        / Update registered Galaxy user
    {% else %}
        / Register Galaxy user
    {% endif %}
{% endblock sb_admin_breadcrumb_active %}
{% block panel_title %}Register{% endblock panel_title %}


{% block sb_admin_custom_js %}
    <script type="text/javascript" src="{% static 'admin/js/vendor/jquery/jquery.js' %}"></script>#}
    {{ form.media }}
{% endblock sb_admin_custom_js %}